import { DragModal, DragModalTitle, getComputedHeight } from '@/components/a/drag-modal/index.tsx';
import { useChatStore } from '../store';
import { useShallow } from 'zustand/shallow';
import { Button } from '@/components/a/button.tsx';
import { TextEditor } from '@kevisual/markdown-editor/tiptap/editor.ts';
import { Select } from '@/components/a/select.tsx';
import { useEffect, useRef, useState } from 'react';
import { getSuggestionItems } from '../editor/suggestion/item';
import { html2md } from '@kevisual/markdown-editor/tiptap/index.ts';
import { chatId } from '../utils/uuid';

export const ChatContextDialog = () => {
  const computedHeight = getComputedHeight();
  const { showContext, setShowContext, updateMessage, contextMessage, setContextMessage, setModelId, modelId } = useChatStore(
    useShallow((state) => ({
      showContext: state.showContext,
      setShowContext: state.setShowContext,
      updateMessage: state.updateMessage,
      contextMessage: state.contextMessage,
      setContextMessage: state.setContextMessage,
      setModelId: state.setModelId,
      modelId: state.modelId,
    })),
  );

  const ref = useRef<HTMLDivElement>(null);
  const editorRef = useRef<TextEditor | null>(null);
  useEffect(() => {
    if (ref.current) {
      editorRef.current = new TextEditor();
      editorRef.current.createEditor(ref.current, {
        items: getSuggestionItems(),
      });
    }
    return () => {
      if (ref.current && editorRef.current) {
        editorRef.current?.destroy?.();
      }
    };
  }, []);
  useEffect(() => {
    if (showContext && contextMessage?.content) {
      contentToEditor(contextMessage.content);
      contextMessage?.role && setRole(contextMessage.role);
    } else if (showContext && typeof contextMessage === 'undefined') {
      editorRef.current?.setContent('');
    }
  }, [showContext]);
  const contentToEditor = async (content: string) => {
    editorRef.current?.setContent(content);
  };
  const onSave = async () => {
    const hasId = contextMessage?.id;
    const html = editorRef.current?.getHtml()!;
    const md = await html2md(html);
    const newMessage = {
      ...contextMessage,
      role: role,
      id: hasId || chatId(),
      content: md,
      name: 'user',
    };
    if (!hasId) {
      newMessage.createdAt = new Date().getTime();
    }
    newMessage.updatedAt = new Date().getTime();
    updateMessage(newMessage);
    setShowContext(false);
  };
  const [role, setRole] = useState<string>('user');
  return (
    <DragModal
      focus={modelId === 'chat-context'}
      title={
        <DragModalTitle
          title='Markdown编辑器'
          onClick={() => {
            setModelId('chat-context');
          }}
          onClose={() => {
            setShowContext(false);
          }}
        />
      }
      containerClassName={showContext ? '' : 'hidden'}
      content={
        <div className='w-full h-full p-2 '>
          <div className='text-sm text-gray-500 border-b border-gray-200 pb-2 flex items-center gap-2'>
            角色:
            <Select
              className='w-40 ml-2'
              size='small'
              value={role}
              onChange={(e) => {
                setRole(e);
              }}
              options={[
                { label: 'User', value: 'user' },
                { label: 'Assistant', value: 'assistant' },
                { label: 'System', value: 'system' },
                { label: 'Tools', value: 'tool' },
              ]}
            />
          </div>
          <div className='w-full scrollbar' style={{ height: 'calc(100% - 90px)' }} ref={ref}></div>
          <div className='flex justify-end gap-2 ' style={{ height: '40px' }}>
            <Button color='primary' onClick={onSave}>
              保存
            </Button>
            <Button onClick={() => setShowContext(false)}>取消</Button>
          </div>
        </div>
      }
      defaultSize={{
        width: 600,
        height: 400,
      }}
      style={{
        left: computedHeight.width / 2 - 300,
        top: computedHeight.height / 2 - 200,
      }}
    />
  );
};
