import { useChatStore } from '../store';
import { useShallow } from 'zustand/shallow';
import { IconButton } from '@/components/a/button.tsx'
import { Bug, BugOff, Copy, Download, RotateCcw, Upload, X } from 'lucide-react';
import { Divider } from '@/components/a/divider.tsx'
import { Tooltip } from '@/components/a/tooltip.tsx'
import { copyText } from '../utils/copy';
import { useEffect, useRef, useState } from 'react';
import { getCodeTemplate, runCode } from '../store/generte-text';
import clsx from 'clsx';

export type SelfUsage = {
  group: string;
  model: string;
  token: number;
  [key: string]: any;
};
export type UsageData = {
  rootUsage?: {
    token?: number;
  };
  selfUsage?: SelfUsage[];
};
export const ChatCopyList = () => {
  const store = useChatStore(
    useShallow((state) => ({
      currentUserModel: state.currentUserModel,
      setCurrentUserModel: state.setCurrentUserModel,
      modelList: state.modelList,
      clearConfigCache: state.clearConfigCache,
      showSetting: state.showSetting,
      getModelList: state.getModelList,
      messages: state.messages,
    })),
  );
  const contentEditableRef = useRef<HTMLDivElement>(null);
  const [text, setText] = useState('');
  const [mode, setMode] = useState<'edit' | 'copy'>('edit');
  const [code, setCode] = useState('');
  useEffect(() => {
    generte();
  }, [store.messages, code]);
  useEffect(() => {
    const code = localStorage.getItem('code-copy-template');
    const cacheMode = localStorage.getItem('code-copy-mode');
    if (code) {
      setCode(code);
    } else {
      setCode(getCodeTemplate());
    }
    if (cacheMode) {
      setMode(cacheMode as 'edit' | 'copy');
    }
  }, []);
  const generte = async () => {
    if (!contentEditableRef.current) return;
    if (!code) return;
    try {
      const jsModule = await runCode(code);
      const result = await jsModule.generateText(store.messages);
      setText(result);
      contentEditableRef.current.innerHTML = result;
    } catch (error) {
      contentEditableRef.current.innerHTML = 'error run code';
    }
  };
  const setCodeTemplate = () => {
    const codeTemplate = getCodeTemplate();
    localStorage.setItem('code-copy-template', codeTemplate);
    setCode(codeTemplate);
  };
  const changeMode = () => {
    setMode(mode === 'edit' ? 'copy' : 'edit');
    localStorage.setItem('code-copy-mode', mode === 'edit' ? 'copy' : 'edit');
  };
  return (
    <div className='w-full h-full p-2'>
      <div className='pt-1 flex gap-2 pb-4'>
        <Tooltip title='复制'>
          <IconButton
            onClick={() => {
              copyText(text);
            }}>
            <Copy className='w-4 h-4' />
          </IconButton>
        </Tooltip>
        <Tooltip title='导出json'>
          <IconButton>
            <Download className='w-4 h-4' />
          </IconButton>
        </Tooltip>
        <Tooltip title='导入json'>
          <IconButton>
            <Upload className='w-4 h-4' />
          </IconButton>
        </Tooltip>
        <Divider orientation='vertical' />
        <Tooltip title={mode === 'edit' ? '编辑导出模式' : '预览模式'}>
          <IconButton
            onClick={() => {
              changeMode();
            }}>
            {mode === 'edit' ? <Bug className='w-4 h-4' /> : <BugOff className='w-4 h-4' />}
          </IconButton>
        </Tooltip>
        <Tooltip title='恢复默认的代码配置'>
          <IconButton onClick={() => setCodeTemplate()}>
            <RotateCcw className='w-4 h-4' />
          </IconButton>
        </Tooltip>
      </div>
      <Divider />
      <div
        className='flex gap-2 py-4 text-sm scrollbar overflow-y-auto'
        style={{
          height: 'calc(100% - 40px)',
        }}>
        <div
          className={clsx('w-full h-full whitespace-pre-wrap scrollbar', mode === 'edit' ? 'flex-1' : '')}
          contentEditable
          style={{
            outline: 'none',
            overflowY: 'auto',
          }}
          ref={contentEditableRef}></div>
        <div className={clsx('w-full h-full whitespace-pre-wrap', mode === 'edit' ? 'flex-1' : 'hidden')}>
          <Tooltip title='自动缓存到code-copy-template中'>
            <div className='text-sm w-[140px] flex gap-2 text-gray-500 pb-2 items-center'>
              编辑导出模版
              <div className='p-0.5 hover:bg-gray-100 rounded-md cursor-pointer' onClick={() => changeMode()}>
                <X className='w-4 h-4' />
              </div>
            </div>
          </Tooltip>
          <textarea
            className='w-full border scrollbar border-gray-200 rounded-md shadow-sm p-4'
            value={code}
            onChange={(e) => {
              localStorage.setItem('code-copy-template', code);
              setCode(e.target.value);
            }}
            style={{
              height: 'calc(100% - 40px)',
              outline: 'none',
              overflowY: 'auto',
            }}></textarea>
        </div>
      </div>
    </div>
  );
};
