import { useChatStore } from '../store';
import { useShallow } from 'zustand/shallow';
import { IconButton } from '@/components/a/button.tsx';
import { BookmarkX, CircleCheckBig, Copy, Edit, LassoSelectIcon, RefreshCcw, Trash } from 'lucide-react';
import clsx from 'clsx';
import { Divider } from '@/components/a/divider.tsx'
import { Tooltip } from '@/components/a/tooltip.tsx'
import { useEffect, useState } from 'react';
import { queryChat } from '../store';
import { toast } from 'react-toastify';
import prettyBytes from 'pretty-bytes';
import { copyText } from '../utils/copy';
export type SelfUsage = {
  group: string;
  model: string;
  token: number;
  [key: string]: any;
};
export type UsageData = {
  rootUsage?: {
    token?: number;
  };
  selfUsage?: SelfUsage[];
};
export const ChatSettingList = () => {
  const store = useChatStore(
    useShallow((state) => ({
      currentUserModel: state.currentUserModel,
      setCurrentUserModel: state.setCurrentUserModel,
      modelList: state.modelList,
      clearConfigCache: state.clearConfigCache,
      showSetting: state.showSetting,
      getModelList: state.getModelList,
      messages: state.messages,
    })),
  );
  const [usageData, setUsageData] = useState<UsageData>({});
  useEffect(() => {
    if (store.showSetting) {
      getUsage();
    }
  }, [store.showSetting]);
  const getUsage = async () => {
    const res = await queryChat.getChatUsage();
    console.log(res);
    if (res.code === 200) {
      setUsageData(res.data);
    } else {
      toast.error(res.message);
    }
  };
  const clearSelfUsage = async () => {
    const res = await queryChat.clearSelfUsage();
    if (res.code === 200) {
      toast.success(res.message);
    } else {
      toast.error(res.message);
    }
  };
  return (
    <div className='w-full h-full p-2'>
      <div className='pt-1 flex gap-2 pb-4'>
        <Tooltip title='刷新本地模型配置项, 默认会缓存到浏览器。'>
          <IconButton onClick={() => store.getModelList(true)}>
            <RefreshCcw className='w-4 h-4' />
          </IconButton>
        </Tooltip>
        <Divider orientation='vertical' />
        <Tooltip title='复制所有的消息'>
          <IconButton
            onClick={() => {
              console.log(store.messages);
              copyText(JSON.stringify(store.messages));
            }}>
            <Copy className='w-4 h-4' />
          </IconButton>
        </Tooltip>
        <Divider orientation='vertical' />
        <Tooltip title='清除自己的服务器缓存配置项'>
          <IconButton
            onClick={() => {
              store.clearConfigCache();
            }}>
            <BookmarkX className='w-4 h-4' />
          </IconButton>
        </Tooltip>
        <Tooltip title='清除当前自己模型的统计'>
          <IconButton
            onClick={() => {
              clearSelfUsage();
            }}>
            <Trash className='w-4 h-4' />
          </IconButton>
        </Tooltip>
      </div>
      <Divider />
      <div
        className='flex flex-col gap-2 py-2 text-sm scrollbar overflow-y-auto'
        style={{
          height: 'calc(100% - 40px)',
        }}>
        <div>使用root的模型统计: token: {prettyBytes(usageData.rootUsage?.token || 0)}</div>
        <div className='py-2'>
          自己模型使用统计
          <div className='flex flex-col gap-2 pl-4 mt-3'>
            {usageData.selfUsage?.map((item) => {
              const keys = Object.keys(item);
              const defaultKeys = ['model', 'token', 'group'];
              const days = keys.filter((key) => !defaultKeys.includes(key));
              return (
                <div key={item.model} className='flex items-center gap-2'>
                  <div className='text-sm shadow-sm p-2 rounded-md w-full border border-gray-200'>
                    <div className='font-bold'>group: {item.group}</div>
                    <div className=' flex gap-2 items-center'>
                      <div className=' w-2 h-2 bg-secondary rounded-full' /> {item.model}
                    </div>
                    <div>token: {prettyBytes(item.token)}</div>
                    <div className='text-xs text-gray-500'>
                      {days.map((day) => {
                        return (
                          <div key={day}>
                            {day}: {item[day] || 0} 次
                          </div>
                        );
                      })}
                    </div>
                  </div>
                </div>
              );
            })}
          </div>
        </div>
      </div>
    </div>
  );
};
