import { ResponseMarkdown, Markdown } from '@/components/ai/RenderMarkdown';
import clsx from 'clsx';
import { Actions } from './actions/Actions';
import { ChastHistoryMessage } from '../store/type';
import { useChatStore } from '../store';
import { useShallow } from 'zustand/shallow';
import { IconButton } from '@/components/a/button';
import { Eraser, Move, Plus } from 'lucide-react';
import { Tooltip } from '@/components/a/tooltip';
import { ReactSortable } from 'react-sortablejs';
import { useEffect, useMemo } from 'react';

export type MessageData = {
  content: string;
  role: 'user' | 'assistant' | 'system';
  response?: Response | null;
  className?: string;
  onFinish?: (text: string) => void;
  // onFinish之前，每200ms调用一次
  onChange?: (text: string) => void;
  id?: string;
  randomId?: string;
};
export type MessageProps = { data: ChastHistoryMessage } & MessageData;
export const AssistantMessage = (props: MessageProps) => {
  if (props.response) {
    return <ResponseMarkdown className={clsx(props.className)} onChange={props.onChange} response={props.response} onFinish={props.onFinish} />;
  }
  return <Markdown className={clsx(props.className)} markdown={props.content} />;
};

export const Message = (props: MessageProps) => {
  const isUser = props.role === 'user';
  const isHide = !!props.data.hide;
  const isNoUse = !!props.data.noUse;
  const { showAllContext, updateMessage } = useChatStore(
    useShallow((state) => {
      return {
        showAllContext: state.showAllContext,
        updateMessage: state.updateMessage,
      };
    }),
  );
  if (!showAllContext && isHide) {
    // return <div className=''></div>;
    return null;
  }
  const isNotShow = !showAllContext && isHide;
  const isContext = !isHide && !isNoUse;
  const onNoUseChange = () => {
    const data = props.data;
    const newData = { ...data, noUse: !data.noUse };
    updateMessage(newData);
  };
  // const imgUrl = '/root/center/panda.jpg'
  const imgUrl = 'https://kevisual.xiongxiao.me/root/center/panda.jpg';
  return (
    <div className={clsx('py-2  px-2 mt-1', props.className, isContext && 'shadow-md border-b border-gray-300 box-content rounded-sm', isNotShow && 'hidden')}>
      <div className={clsx('chat-message')}>
        <div
          className={clsx(
            {
              'message-user': isUser,
              'message-assistant': !isUser,
            },
            'flex flex-col',
          )}>
          <div>
            <div className=' flex gap-4 group items-center'>
              <div
                className='cursor-pointer'
                onClick={() => {
                  onNoUseChange();
                }}>
                <div className={clsx('message-avatar flex gap-3 items-center py-2', isUser && 'hidden')}>
                  <img src={imgUrl} alt='' className='w-6 h-6 rounded-full' />
                  <div className='' style={{ textTransform: 'capitalize' }}>
                    {props.role}
                  </div>
                </div>
                <div className={clsx('message-avatar flex gap-3 items-center py-2', !isUser && 'hidden')}>
                  <img src={imgUrl} alt='' className='w-6 h-6 rounded-full' />
                  <div
                    className=''
                    style={{
                      textTransform: 'capitalize',
                    }}>
                    {props.role}
                  </div>
                </div>
              </div>
              <div className='message-move hidden group-hover:block'>
                <div className='flex gap-2 cursor-pointer'>
                  <Move className='w-4 h-4' />
                </div>
              </div>
            </div>
            <div className={clsx('message-content scrollbar overflow-x-hidden', isHide && 'opacity-50')}>
              <AssistantMessage {...props} className='p-2 rounded' />
            </div>
          </div>
        </div>
      </div>
      <Actions data={props.data} />
    </div>
  );
};
export const Messages = (props: { messages: ChastHistoryMessage[] }) => {
  const { messages } = props;
  const { setMessages, setContextMessage, setShowContext, showAllContext } = useChatStore(
    useShallow((state) => {
      return {
        showAllContext: state.showAllContext,
        setMessages: state.setMessages,
        setContextMessage: state.setContextMessage,
        setShowContext: state.setShowContext,
      };
    }),
  );

  const onEraser = () => {
    const newMessages = messages.map((message) => {
      return {
        ...message,
        noUse: true,
      };
    });
    setMessages([...newMessages]);
  };
  const list = messages.filter((message) => {
    const isHide = !!message.hide;
    return !(!showAllContext && isHide);
  });
  const reactSortable = useMemo(() => {
    return (
      <ReactSortable
        handle='.message-move'
        list={list as any}
        setList={(newList) => {}}
        onEnd={(event) => {
          const { newIndex, oldIndex } = event;
          const newMessages = [...messages];
          // @ts-ignore
          const _oldIndex = messages.findIndex((item) => item.id === list[oldIndex].id);
          // @ts-ignore
          const _newIndex = messages.findIndex((item) => item.id === list[newIndex].id);
          const tmp = newMessages[_oldIndex];
          newMessages[_oldIndex] = newMessages[_newIndex];
          newMessages[_newIndex] = tmp;
          setMessages(newMessages);
        }}
        direction={'vertical'}
        swap={true}
        chosenClass='sortable-chosen'
        dragClass='sortable-drag'
        swapThreshold={0.6}
        animation={200}>
        {messages.map((message) => {
          const id = message.id;
          const onChange = (text: string) => {};
          const onFinish = (text: string) => {};
          return (
            <Message
              className='message-item'
              content={message.content}
              role={message.role as any}
              data={message}
              id={id}
              key={id + String(message.updatedAt)}
              onChange={onChange}
              onFinish={onFinish}
            />
          );
        })}
      </ReactSortable>
    );
  }, [messages, list]);
  return (
    <>
      {reactSortable}
      <div className='flex py-2 gap-2'>
        <Tooltip title='添加新的上下文'>
          <IconButton
            onClick={() => {
              setContextMessage(undefined);
              setShowContext(true);
            }}>
            <Plus className='w-4 h-4' />
          </IconButton>
        </Tooltip>
        <Tooltip title='清理所有上下文'>
          <IconButton onClick={onEraser}>
            <Eraser className='w-4 h-4' />
          </IconButton>
        </Tooltip>
      </div>
    </>
  );
};
