import { StoreManager } from '@kevisual/store';
import { useContextKey } from '@kevisual/store/context';
import { StateCreator } from 'zustand';
import { query } from '@/modules/query';
import { useStore, BoundStore } from '@kevisual/store/react';
import { toast } from 'react-toastify';
import { ChastHistoryMessage } from './type';
import { CacheStore } from '@kevisual/cache/cache-store';
import { QueryMark } from '@/query/query-mark/query-mark';
import { QueryApp as QueryAi } from '@/query/query-ai/query-ai';
export const queryMark = new QueryMark({ query: query, markType: 'chat' });
export const queryChat = new QueryAi({ query });

const dbName = 'chat';
export const store = useContextKey('store', () => {
  return new StoreManager();
});

type ChatStore = {
  id: string;
  setId: (id: string) => void;
  loading: boolean;
  setLoading: (loading: boolean) => void;
  messages: ChastHistoryMessage[];
  updateTime: number;
  setMessages: (messages: ChastHistoryMessage[], needCache?: boolean) => void;

  init: (id: string) => void;
  /**
   * 删除消息
   * @param index 消息索引
   */
  deleteMessage: (id: string) => void;
  /**
   * 清空消息
   */
  clearMessage: () => void;
  /**
   * 更新消息
   * @param message 消息
   */
  updateMessage: (message: ChastHistoryMessage) => void;

  showAllContext: boolean;
  setShowAllContext: (showAllContext: boolean) => void;

  showChatSetting: boolean;
  setShowChatSetting: (showChatSetting: boolean) => void;

  showList: boolean;
  setShowList: (showList: boolean) => void;
  newChat: boolean;
  setNewChat: (newChat: boolean) => void;

  historyList: any[];
  setHistoryList: (historyList: any[]) => void;

  chatData: any;
  setChatData: (chatData: any) => void;

  getChatList: () => Promise<any>;
  updateChat: (data: any) => Promise<any>;
  deleteChat: (id: string) => Promise<any>;

  // 上下文对话框编辑
  showContext: boolean;
  setShowContext: (showContext: boolean) => void;
  contextMessage?: ChastHistoryMessage;
  setContextMessage: (contextMessage?: ChastHistoryMessage) => void;

  currentUserModel?: { username: string; model: { group: string; provider: string; model: string } };
  setCurrentUserModel: (currentUserModel: { username: string; model: { group: string; provider: string; model: string } }) => void;
  modelList: any[];
  setModelList: (modelList: any[]) => void;
  getModelList: (force?: boolean) => Promise<any>;
  chat: () => Promise<any>;
  clearConfigCache: () => Promise<any>;

  modelId: string;
  setModelId: (modelId: string) => void;

  showSetting: boolean;
  setShowSetting: (showSetting: boolean) => void;

  showCopy: boolean;
  setShowCopy: (showCopy: boolean) => void;
};
export const createChatStore: StateCreator<ChatStore, [], [], ChatStore> = (set, get, store) => {
  return {
    id: '',
    setId: (id: string) => set(() => ({ id })),
    loading: false,
    setLoading: (loading: boolean) => set(() => ({ loading })),
    updateTime: 0,
    messages: [],
    setMessages: (messages: ChastHistoryMessage[], needCache = true) => {
      const id = get().id;
      if (needCache) {
        const chatData = get().chatData;
        if (chatData) {
          const data = chatData.data || {};
          data.messages = messages;
          chatData.data = data;
          const cache = new CacheStore({ dbName });
          cache.set(id, chatData);
        }
      }
      const now = Date.now();
      set(() => ({ updateTime: now }));
      set({ messages });
    },

    init: async (id: string) => {
      if (!id) {
        return;
      }
      // clear default
      const { setMessages } = get();
      setMessages([], false);

      const { getModelList, setCurrentUserModel } = get();
      await getModelList();
      const cache = new CacheStore({ dbName });
      const chatData = await cache.get(id);
      const currentUserModel = await cache.get('currentUserModel');
      if (currentUserModel) {
        setCurrentUserModel(currentUserModel);
      }
      if (chatData) {
        const messages = chatData.data?.messages || [];
        set(() => ({ messages: messages, chatData }));
      }

      const res = await queryMark.getMark(id);

      if (res.code === 200) {
        const resChatData = res.data as any;
        if (chatData && resChatData.updatedAt === chatData.updatedAt) {
          console.log('no update', 'time is ', resChatData.updatedAt, chatData.updatedAt);
          return;
        }
        console.log('update chat data', resChatData);
        get().setMessages(resChatData?.data?.messages || []);
        set(() => ({ chatData: resChatData }));
      } else {
        toast.error(res.message);
      }
    },
    deleteMessage: (id: string) => {
      const { messages, setMessages } = get();
      const newMessages = messages.filter((m) => m.id !== id);
      setMessages(newMessages);
    },
    clearMessage: () => {
      const { setMessages } = get();
      setMessages([]);
    },
    updateMessage: (message: ChastHistoryMessage) => {
      const { messages, setMessages } = get();
      let isNew = true;
      let newMessages = messages.map((m) => {
        if (m.id === message.id) {
          isNew = false;
          return { ...m, ...message, updatedAt: new Date().getTime() };
        }
        return m;
      });
      if (isNew) {
        newMessages.push(message);
      }
      setMessages(newMessages);
    },
    showAllContext: false,
    setShowAllContext: (showAllContext: boolean) => set(() => ({ showAllContext })),
    showChatSetting: false,
    setShowChatSetting: (showChatSetting: boolean) => set(() => ({ showChatSetting })),
    showList: false,
    setShowList: (showList: boolean) => set(() => ({ showList })),
    newChat: false,
    setNewChat: (newChat: boolean) => set(() => ({ newChat })),
    historyList: [],
    setHistoryList: (historyList: any[]) => set(() => ({ historyList })),
    getChatList: async () => {
      const res = await queryMark.getMarkList();
      if (res.code === 200) {
        const len = res.data.list.length;
        // if (len === 0) {
        //   set(() => ({ newChat: true }));
        // }
        set(() => ({ historyList: res.data.list }));
      } else {
        toast.error(res.message || 'request chat list error');
      }
    },
    updateChat: async (data: any) => {
      const res = await queryMark.updateMark(data);
      console.log('update chat res', data);
      if (res.code === 200) {
        toast.success('update success');
        get().setChatData(res.data);
        get().getChatList();
      } else {
        toast.error(res.message || 'error');
      }
      return res;
    },
    deleteChat: async (id: string) => {
      const res = await queryMark.deleteMark(id);
      if (res.code === 200) {
        toast.success('delete success');
        get().getChatList();
      } else {
        toast.error(res.message || 'error');
      }
      return res;
    },
    chatData: {},
    setChatData: (chatData: any) => {
      const cache = new CacheStore({ dbName });
      cache.set(chatData.id, chatData);
      set(() => ({ chatData }));
    },
    showContext: false,
    setShowContext: (showContext: boolean) => set(() => ({ showContext })),
    contextMessage: {
      role: 'user',
      content: '',
      name: 'user',
    },
    setContextMessage: (contextMessage?: ChastHistoryMessage) => set(() => ({ contextMessage })),
    modelList: [],
    setModelList: (modelList: any[]) => set(() => ({ modelList })),
    getModelList: async (force?: boolean) => {
      const cache = new CacheStore({ dbName });
      const modelList = await cache.get('modelList');
      if (modelList && !force) {
        set(() => ({ modelList }));
        return;
      }
      // const res = await queryChat.getModelList();
      // if (res.code === 200) {
      //   set(() => ({ modelList: res.data.list }));
      //   cache.set('modelList', res.data.list);
      // } else {
      //   toast.error(res.message || 'error');
      // }
    },
    currentUserModel: undefined,
    setCurrentUserModel: (currentUserModel: { username: string; model: { group: string; provider: string; model: string } }) => {
      set(() => ({ currentUserModel }));
      const cache = new CacheStore({ dbName });
      cache.set('currentUserModel', currentUserModel);
    },
    chat: async () => {
      const { id, currentUserModel, messages } = get();
      if (!currentUserModel) {
        toast.error('请先设置当前用户模型');
        return;
      }
      const { username, model } = currentUserModel;
      const { group, provider, model: modelName } = model;
      const chatOpts = {
        username,
        group,
        model: modelName,
      };
      set(() => ({ loading: true }));
      const loaded = toast.loading('loading...');
      const res = await queryChat.chat({ id, messages }, chatOpts);
      toast.dismiss(loaded);
      set(() => ({ loading: false }));
      console.log('chat res', res);
      if (res.code === 200) {
        const data = res.data;
        const aiChatHistory = data.aiChatHistory;
        console.log('aiChatHistory ', data);
        if (aiChatHistory) {
          const { messages } = aiChatHistory;
          get().setChatData(aiChatHistory);
          get().setMessages(messages);
          console.log('aiChatHistory update', aiChatHistory);
        } else if (data.message) {
          const message = data.message;
          get().updateMessage(message);
        }
      } else {
        toast.error(res.message || 'error');
      }
    },
    clearConfigCache: async () => {
      const res = await queryChat.clearConfigCache();
      // 清理缓存
      if (res.code === 200) {
        toast.success('clear config cache success');
        setTimeout(() => {
          get().getModelList(true);
        }, 4000);
      } else {
        toast.error(res.message || 'error');
      }
    },
    modelId: '',
    setModelId: (modelId: string) => set(() => ({ modelId })),

    showSetting: false,
    setShowSetting: (showSetting: boolean) => set(() => ({ showSetting })),

    showCopy: false,
    setShowCopy: (showCopy: boolean) => set(() => ({ showCopy })),
  };
};

export const useChatStore = useStore as BoundStore<ChatStore>;
