import { useEffect, useRef, useState } from 'react';
import Draggable from 'react-draggable';
import { cn as clsxMerge } from '@/lib/utils';
import { Resizable } from 're-resizable';
import { X } from 'lucide-react';

type DragModalProps = {
  title?: React.ReactNode;
  content?: React.ReactNode;
  onClose?: () => void;
  containerClassName?: string;
  handleClassName?: string;
  contentClassName?: string;
  focus?: boolean;
  /**
   * 默认大小, 单位为px
   * width: defaultSize.width || 320
   * height: defaultSize.height || 400
   */
  defaultSize?: {
    width: number;
    height: number;
  };
  style?: React.CSSProperties;
};
export const DragModal = (props: DragModalProps) => {
  const dragRef = useRef<HTMLDivElement>(null);

  return (
    <Draggable
      nodeRef={dragRef as any}
      onStop={(e, data) => {
        // console.log(e, data);
      }}
      handle='.handle'
      grid={[1, 1]}
      scale={1}
      bounds='parent'
      defaultPosition={{
        x: 0,
        y: 0,
      }}>
      <div
        className={clsxMerge('absolute top-0 left-0 bg-white rounded-md border border-gray-200 shadow-sm', props.focus ? 'z-30' : '', props.containerClassName)}
        ref={dragRef}
        style={props.style}>
        <div className={clsxMerge('handle cursor-move border-b border-gray-200 py-2 px-4', props.handleClassName)}>{props.title || 'Move'}</div>
        <Resizable
          className={clsxMerge('', props.contentClassName)}
          defaultSize={{
            width: props.defaultSize?.width || 600,
            height: props.defaultSize?.height || 400,
          }}
          onResizeStop={(e, direction, ref, d) => {
            // console.log(e, direction, ref, d);
          }}
          enable={{
            bottom: true,
            right: true,
            bottomRight: true,
          }}>
          {props.content}
        </Resizable>
      </div>
    </Draggable>
  );
};

type DragModalTitleProps = {
  title?: React.ReactNode;
  className?: string;
  onClose?: () => void;
  children?: React.ReactNode;
  onClick?: () => void;
};
export const DragModalTitle = (props: DragModalTitleProps) => {
  return (
    <div
      className={clsxMerge('flex flex-row items-center justify-between', props.className)}
      onClick={(e) => {
        e.stopPropagation();
        props.onClick?.();
      }}>
      <div className='text-sm font-medium text-gray-700'>
        {props.title}
        {props.children}
      </div>
      <div
        className='text-gray-500 cursor-pointer p-2 hover:bg-gray-100 rounded-md'
        onClick={(e) => {
          e.stopPropagation();
          props.onClose?.();
        }}>
        <X className='w-4 h-4 ' />
      </div>
    </div>
  );
};

export const getComputedHeight = () => {
  const height = window.innerHeight;
  const width = window.innerWidth;
  return { height, width };
};

export const useComputedHeight = () => {
  const [computedHeight, setComputedHeight] = useState({
    height: 0,
    width: 0,
  });
  useEffect(() => {
    const height = window.innerHeight;
    const width = window.innerWidth;
    setComputedHeight({ height, width });
  }, []);
  return computedHeight;
};
