## Peerjs 能做什么？

如果不是局域网，使用 **PeerJS** 和 WebRTC 技术仍然可以在互联网上实现点对点（P2P）消息发送，但需要满足以下条件：

---

### **1. 必须有一个信令服务器**
- WebRTC 需要一个 **信令服务器** 用于初始连接的建立。
- **PeerJS Server** 就是一个轻量级的信令服务器，它帮助两个设备（Peers）互相交换连接信息（如 SDP 和 ICE 候选信息）。
- PeerJS Server 本身并不传输实际的数据，只负责建立连接。

---

### **2. 公网穿透问题**
- WebRTC 需要设备之间直接通信（点对点），但在实际网络中，由于 **NAT（网络地址转换）** 和防火墙的限制，设备可能无法直接连接到对方。
- 为了解决这个问题，需要使用 **STUN** 和 **TURN** 服务器：
  1. **STUN 服务器**：帮助设备发现自己的公网 IP 和端口，用于尝试直接连接。
  2. **TURN 服务器**：在无法直接连接时，充当中继服务器传输数据。

PeerJS 支持自定义 STUN/TURN 服务器配置：

```javascript
const peer = new Peer({
  config: {
    iceServers: [
      { url: "stun:stun.l.google.com:19302" }, // 免费的 Google STUN 服务器
      {
        url: "turn:your-turn-server.com",
        username: "user",
        credential: "password"
      }
    ]
  }
});
```

---

### **3. 消息发送的实现**
在非局域网环境中，只要能够建立 WebRTC 连接，消息仍然可以通过 P2P 方式直接发送，而无需依赖中间服务器。

示例代码：

#### 服务端（运行 PeerJS Server）：
```bash
npx peerjs --port 9000 --key myapp
```

#### 客户端 A：
```javascript
const peer = new Peer("peerA", { host: "your-server.com", port: 9000, key: "myapp" });

peer.on("open", (id) => {
  console.log("My peer ID is:", id);

  // 连接到客户端 B
  const conn = peer.connect("peerB");
  conn.on("open", () => {
    conn.send("Hello from peerA!");
  });

  conn.on("data", (data) => {
    console.log("Received:", data);
  });
});
```

#### 客户端 B：
```javascript
const peer = new Peer("peerB", { host: "your-server.com", port: 9000, key: "myapp" });

peer.on("connection", (conn) => {
  conn.on("data", (data) => {
    console.log("Received:", data);
    conn.send("Hello back from peerB!");
  });
});
```

---

### **注意事项**
1. **需要一个稳定的信令服务器**：PeerJS Server 是建立连接的前提。
   - 可以使用官方的 `peerjs` 服务，也可以自行托管。
2. **STUN/TURN 服务是关键**：
   - 公网环境中，推荐使用免费的 STUN 服务器，例如 Google STUN (`stun:stun.l.google.com:19302`)。
   - 对于企业或大规模应用，建议搭建自己的 TURN 服务器，例如使用 [Coturn](https://github.com/coturn/coturn)。
3. **网络质量和带宽**：
   - P2P 消息的传输速度取决于两端的网络质量。
   - 数据量较大时（如视频流），需要优化传输。

---

### **总结**
在非局域网环境中，通过 **PeerJS** 和 WebRTC，配合信令服务器（PeerJS Server）和 STUN/TURN 服务器，你可以实现点对点消息传递。只要配置正确，消息和数据可以直接在设备之间传输，而不需要经过中间服务器。



**PeerJS** 是一个用于构建 WebRTC 应用程序的 JavaScript 库，提供了简单的接口来创建点对点（P2P）连接。使用 PeerJS 以及它的服务器（PeerServer），你可以开发许多有趣的实时应用程序，以下是一些可能的项目想法：

---

### **1. 实时视频/语音聊天**
- **应用场景**：创建类似 Zoom 或 Google Meet 的视频通话应用。
- **功能**：
  - 支持一对一或多对多的视频通话。
  - 基于 WebRTC 的低延迟音视频传输。
  - 使用 PeerJS Server 管理 Peer ID 和信令。

---

### **2. 文件共享应用**
- **应用场景**：开发一个无需上传到服务器的点对点文件共享工具。
- **功能**：
  - 直接通过 P2P 传输文件（大文件可能需要分块传输）。
  - 支持拖拽式文件发送。
  - 使用加密保证传输安全性。

---

### **3. 实时协作工具**
- **应用场景**：类似 Google Docs 的实时协作文档编辑器。
- **功能**：
  - 多人实时同步文本、图像、代码等内容。
  - 点对点传输减少延迟，提高效率。
  - 可结合 WebSocket 提供信令服务器。

---

### **4. 多人在线游戏**
- **应用场景**：开发一个轻量级的多人在线游戏（如象棋、贪吃蛇）。
- **功能**：
  - 使用 PeerJS 建立玩家之间的 P2P 连接。
  - 每个玩家充当主机同步游戏状态。
  - 游戏状态通过 P2P 广播到所有玩家。

---

### **5. 局域网聊天工具**
- **应用场景**：开发一个无需互联网的局域网聊天应用。
- **功能**：
  - 使用局域网 P2P 连接直接聊天。
  - 支持群聊和私聊功能。
  - 可选加密消息。

---

### **6. 实时屏幕共享**
- **应用场景**：创建一个支持屏幕共享的在线演示工具。
- **功能**：
  - 基于 WebRTC 的屏幕共享功能。
  - 观众通过 PeerJS 链接直接观看共享屏幕。
  - 支持音频同步。

---

### **7. 分布式计算**
- **应用场景**：开发一个利用用户设备的分布式计算工具。
- **功能**：
  - 点对点分享计算任务。
  - 使用 PeerJS 传输计算任务和结果。
  - 每个节点（Peer）负责部分计算。

---

### **8. 数据同步工具**
- **应用场景**：开发类似 Resilio Sync 的文件同步工具。
- **功能**：
  - 文件夹之间的点对点同步。
  - 断点续传功能。
  - 使用信令服务器管理 Peer 之间的连接。

---

### **开发 PeerJS 应用需要的技术栈**
1. **前端**：
   - HTML/CSS/JavaScript
   - PeerJS 客户端库
2. **后端**（可选）：
   - 使用 PeerJS Server 作为信令服务器。
   - 结合 Node.js 实现 Peer ID 管理或其他功能。
3. **基础设施**：
   - 部署 PeerServer（例如：`peerjs --port 9000`）。
   - 可选 WebSocket 或 STUN/TURN 服务支持。

通过 PeerJS，你可以快速搭建小型实时 P2P 应用，无需复杂的后端开发。结合现有的浏览器 API（如 WebRTC、File API 等），还能实现更加创新的功能。